package net.wanji.utc.hisense.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.CrossSchedulesPO;
import net.wanji.databus.dto.*;
import net.wanji.databus.po.CoordinationStatus;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.CrossSchemeRingsToStagesPO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.databus.po.CrossSchemeRings;
import net.wanji.utc.hisense.service.StaticInfoService;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import javax.ws.rs.core.MediaType;
import java.util.List;

/**
 * 静态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "静态信息接口", tags = "静态信息接口")
@RequestMapping("/staticInfo")
@RestController
@RequiredArgsConstructor
public class StaticInfoController {

    private final StaticInfoService staticInfoService;

    @AspectLog(description = "根据厂商查询信号机信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/crossBasicInfo",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "根据厂商查询信号机信息", notes = "根据厂商查询信号机信息",
            response = SchemePhaseLightsVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemePhaseLightsVO.class)
    })
    public JsonViewObject crossBasicInfo(@RequestBody @Validated CrossInfoDTO crossInfoDTO) throws Exception {
        List<CrossInfoPO> result = staticInfoService.crossBasicInfo(crossInfoDTO);
        return JsonViewObject.newInstance().success(result);
    }

    @AspectLog(description = "方案数据-方案信息、相位信息、灯组信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/schemePhaseLights",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "方案数据-方案信息、相位信息、灯组信息", notes = "方案数据-方案信息、相位信息、灯组信息",
            response = SchemePhaseLightsVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemePhaseLightsVO.class)
    })
    public JsonViewObject schemePhaseLights(@RequestBody @Validated SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        SchemePhaseLightsVO schemePhaseLightsVO = staticInfoService.schemePhaseLights(schemePhaseLightsDTO);
        return JsonViewObject.newInstance().success(schemePhaseLightsVO);
    }

    @AspectLog(description = "计划数据-计划信息、时段信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/planSection",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "计划数据-计划信息、时段信息", notes = "计划数据-计划信息、时段信息", response = PlanSectionVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = PlanSectionVO.class)
    })
    public JsonViewObject planSection(@RequestBody @Validated PlanSectionDTO planSectionDTO) throws Exception {
        List<PlanSectionVO> planSectionVOS = staticInfoService.planSection(planSectionDTO);
        return JsonViewObject.newInstance().success(planSectionVOS);
    }

    @AspectLog(description = "时间表数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/crossSchedules",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "时间表数据", notes = "时间表数据", response = CrossSchedulesPO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchedulesPO.class)
    })
    public JsonViewObject crossSchedules(@RequestBody @Validated CrossSchedulesDTO crossSchedulesDTO) throws Exception {
        List<CrossSchedulesPO> crossSchedulesPOS = staticInfoService.crossSchedules(crossSchedulesDTO);
        return JsonViewObject.newInstance().success(crossSchedulesPOS);
    }

    @AspectLog(description = "5.7协调状态-获取当前方案号", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/crossSchemeNo")
    @ApiOperation(value = "5.7协调状态-获取当前方案号", notes = "协调状态-获取当前方案号", response = CoordinationStatus.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CoordinationStatus.class)
    })
    public JsonViewObject crossPlanId(@NotNull String crossId) throws Exception {
        CoordinationStatus coordinationStatus = staticInfoService.crossPlan(crossId);
        if (ObjectUtils.isNotEmpty(coordinationStatus)) {
            return JsonViewObject.newInstance().success(coordinationStatus);
        } else {
            return JsonViewObject.newInstance().fail("获取当前方案号 失败");

        }
    }

    @AspectLog(description = "5.15. 方案环图请求-根据方案号获取配时方案环图", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/crossSchemeRings")
    @ApiOperation(value = "5.15. 方案环图请求-根据方案号获取配时方案环图", notes = "方案环图请求-根据方案号获取配时方案环图", response = CrossSchemeRings.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchemeRings.class)
    })
    public JsonViewObject crossSchemeRings(@RequestBody @Validated CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        CrossSchemeRings crossSchemeRings = staticInfoService.crossSchemeRings(crossSchemeRingsDTO);
        if (ObjectUtils.isNotEmpty(crossSchemeRings)) {
            return JsonViewObject.newInstance().success(crossSchemeRings);
        } else {
            return JsonViewObject.newInstance().fail("请求海信 方案环图请求 失败");

        }
    }

    @AspectLog(description = "查询方案优化记录", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/schemeOptLog")
    @ApiOperation(value = "查询方案优化记录", notes = "查询方案优化记录", response = CrossSchemeRings.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchemeRings.class)
    })
    public JsonViewObject schemeOptLog(@RequestBody @Validated CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        return JsonViewObject.newInstance().success(staticInfoService.schemeOptLog(crossSchemeRingsDTO));
    }

    @AspectLog(description = "根据时间和路口号查询方案号", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/selectSchemeByParams")
    @ApiOperation(value = "根据时间和路口号查询方案号", notes = "查询方案优化记录", response = CrossSchemeRings.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchemeRings.class)
    })
    public JsonViewObject selectSchemeByParams(@RequestBody @Validated QueryByCrossIdAndTimeDTO queryByCrossIdAndTimeDTO) throws Exception {
        return JsonViewObject.newInstance().success(staticInfoService.selectSchemeByParams(queryByCrossIdAndTimeDTO));
    }

    @AspectLog(description = "根据路口号和方案号查询环转阶段", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/selectCrossSchemeRingsToStages")
    @ApiOperation(value = "根据路口号和方案号查询环转阶段", notes = "根据路口号和方案号查询环转阶段", response = CrossSchemeRings.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchemeRings.class)
    })
    public JsonViewObject selectCrossSchemeRingsToStages(@RequestBody @Validated CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        if (ObjectUtils.isEmpty(crossSchemeRingsDTO.getPattern())) {
            return JsonViewObject.newInstance().fail("方案号不能为空");
        }
        CrossSchemeRingsToStagesPO crossSchemeRingsToStagesPO = staticInfoService.selectCrossSchemeRingsToStages(crossSchemeRingsDTO);
        if (ObjectUtils.isEmpty(crossSchemeRingsToStagesPO)) {
            return JsonViewObject.newInstance().fail("获取海信方案环图失败");
        }
        return JsonViewObject.newInstance().success(crossSchemeRingsToStagesPO);
    }
}
